package order

import (
	logger "eta/eta_mini_ht_api/common/component/log"
	"eta/eta_mini_ht_api/common/exception"
	"eta/eta_mini_ht_api/common/utils/page"
	orderDao "eta/eta_mini_ht_api/models/order"
	"fmt"
	"math/rand"
	"time"
)

type ProductOrderDTO struct {
	ID                int
	OrderID           string
	UserID            int
	ProductID         int
	TotalAmount       string
	ProductPrice      string
	ProductName       string
	PaymentWay        string
	PaymentTimeRemain int64
	Status            string
	CreatedTime       string
}

type ProductOrderDetailDTO struct {
	ProductOrderDTO
	TransactionID int
	PaymentTime   string
	RefundStatus  string
	Remark        string
}

var (
	transProductOrderStatusMap = map[orderDao.OrderStatus]string{
		orderDao.OrderStatusClosed:  "已关闭",
		orderDao.OrderStatusPaid:    "已支付",
		orderDao.OrderStatusPending: "待支付",
		orderDao.OrderStatusRefund:  "售后",
	}

	transRefundStatusMap = map[orderDao.RefundStatus]string{
		orderDao.RefundStatusPending:    "待退款",
		orderDao.RefundStatusFailed:     "退款失败",
		orderDao.RefundStatusProcessing: "退款中",
		orderDao.RefundStatusSuccess:    "退款成功",
	}

	productOrderStatusMap = map[string]orderDao.OrderStatus{
		"closed":  orderDao.OrderStatusClosed,
		"paid":    orderDao.OrderStatusPaid,
		"pending": orderDao.OrderStatusPending,
		"refund":  orderDao.OrderStatusRefund,
	}
)

func GenerateProductOrderNo() string {
	timestamp := time.Now().UnixNano() / 1000000 // 毫秒级时间戳
	// 生成随机数
	rand.New(rand.NewSource(time.Now().UnixNano()))
	randomPart := rand.Intn(999999)
	// 格式化订单号
	orderNumber := fmt.Sprintf("P%d%06d", timestamp, randomPart)
	return orderNumber
}

func CreateProductOrder(orderDTO ProductOrderDTO) (orderNo string, err error) {
	order := convertProductOrder(orderDTO)
	orderNo, err = orderDao.CreateProductOrder(order)
	return
}

func convertProductOrderDTO(order orderDao.ProductOrder) (orderDTO ProductOrderDTO) {
	orderDTO = ProductOrderDTO{
		ID:          order.ID,
		OrderID:     order.OrderID,
		UserID:      order.UserID,
		ProductID:   order.ProductID,
		TotalAmount: order.TotalAmount,
		PaymentWay:  string(order.PaymentWay),
		Status:      transProductOrderStatusMap[order.Status],
		CreatedTime: order.CreatedTime.Format(time.DateTime),
	}
	if order.Status == orderDao.OrderStatusRefund {
		orderDTO.Status = transRefundStatusMap[order.RefundStatus]
	}
	if order.Status == orderDao.OrderStatusPending {
		duration := time.Now().Sub(order.CreatedTime)
		timeout := duration - 15*time.Minute
		if timeout > 0 {
			logger.Info("订单已超时:%v", order.OrderID)
			orderDTO.Status = transProductOrderStatusMap[orderDao.OrderStatusClosed]
		} else {
			orderDTO.PaymentTimeRemain = int64(duration.Seconds())
		}
	}
	return
}
func convertProductOrderDetailDTO(order orderDao.ProductOrder) ProductOrderDetailDTO {
	return ProductOrderDetailDTO{
		ProductOrderDTO: ProductOrderDTO{
			ID:          order.ID,
			OrderID:     order.OrderID,
			UserID:      order.UserID,
			ProductID:   order.ProductID,
			TotalAmount: order.TotalAmount,
			PaymentWay:  string(order.PaymentWay),
			Status:      transProductOrderStatusMap[order.Status],
			CreatedTime: order.CreatedTime.Format(time.DateTime),
		},
		TransactionID: order.TransactionID,
		PaymentTime:   order.PaymentTime.Format(time.DateTime),
		RefundStatus:  string(order.RefundStatus),
		Remark:        order.Remark,
	}
}

func convertProductOrder(order ProductOrderDTO) orderDao.ProductOrder {
	return orderDao.ProductOrder{
		ID:          order.ID,
		OrderID:     order.OrderID,
		UserID:      order.UserID,
		ProductID:   order.ProductID,
		TotalAmount: order.TotalAmount,
	}
}

func GetTotalPageCountByUserId(userId int, orderStatus string) (total int64, latestId int64) {
	var orderStatusEnum orderDao.OrderStatus
	if orderStatus != "" {
		orderStatusEnum = productOrderStatusMap[orderStatus]
		if string(orderStatusEnum) == "" {
			//err := exception.New(exception.IllegalOrderStatus)
			logger.Error("订单状态不合法:%v", orderStatus)
			return
		}
	}
	return orderDao.GetTotalPageCountByUserId(userId, orderStatusEnum)
}
func GetOrderPage(pageInfo page.PageInfo, userId int, orderStatus string) (orderDTOS []ProductOrderDTO, err error) {
	var orderStatusEnum orderDao.OrderStatus
	if orderStatus != "" {
		orderStatusEnum = productOrderStatusMap[orderStatus]
		if string(orderStatusEnum) == "" {
			err = exception.New(exception.IllegalOrderStatus)
			return
		}
	}
	offset := page.StartIndex(pageInfo.Current, pageInfo.PageSize)
	orderList, err := orderDao.GetOrderPage(pageInfo.LatestId, offset, pageInfo.PageSize, userId, orderStatusEnum)
	if err != nil {
		logger.Error("分页查询订单列表失败:%v", err)
		return
	}
	for _, order := range orderList {
		orderDTOS = append(orderDTOS, convertProductOrderDTO(order))
	}
	return
}

func GetOrderDetail(orderId int, userId int) (orderDTO ProductOrderDetailDTO, err error) {
	order, err := orderDao.GetOrderDetail(orderId, userId)
	if err != nil {
		logger.Error("查询订单详情失败:%v", err)
		return
	}
	orderDTO = convertProductOrderDetailDTO(order)
	return
}

func BatchCloseOrder() (err error) {
	//total, minId := orderDao.GetUnPendingOrderCount()
	//if total > 500 {
	//	chunk := (total + 99) / 100
	//	var wg sync.WaitGroup
	//	wg.Add(chunk)
	//	for i := 0; i < chunk; i++ {
	//		go func() {
	//			defer wg.Done()
	//			var ids []int
	//			ids, minId = orderDao.GetPendingOrderIds(minId, 500)
	//			err = orderDao.batchCloseOrder(ids)
	//		}()
	//	}
	//}
	return nil
}
