package order

import (
	"errors"
	logger "eta/eta_mini_ht_api/common/component/log"
	"eta/eta_mini_ht_api/models"
	"github.com/go-sql-driver/mysql"
	"gorm.io/gorm"
	"time"
)

type OrderStatus string
type RefundStatus string
type PaymentStatus string

const (
	//订单状态
	OrderStatusPending    OrderStatus = "pending"
	OrderStatusProcessing OrderStatus = "processing"
	OrderStatusPaid       OrderStatus = "paid"
	OrderStatusClosed     OrderStatus = "closed"
	OrderStatusRefund     OrderStatus = "refund"
	//退款状态
	RefundStatusSuccess    RefundStatus = "success"
	RefundStatusPending    RefundStatus = "pending"
	RefundStatusFailed     RefundStatus = "failed"
	RefundStatusProcessing RefundStatus = "processing"

	WechatPayWay PaymentStatus = "wechat"
	AliPayWay    PaymentStatus = "alipay"
)

type ProductOrder struct {
	ID            int           `gorm:"column:id;primaryKey;autoIncrement" json:"id"`
	OrderID       string        `gorm:"column:order_id;size:255;comment:'订单编号'" json:"order_id"`
	UserID        int           `gorm:"column:user_id;default:null;comment:'用户id'" json:"user_id"`
	ProductID     int           `gorm:"column:product_id;default:null;comment:'产品id'" json:"product_id"`
	TotalAmount   string        `gorm:"column:total_amount;size:255;default:null;comment:'金额'" json:"total_amount"`
	TransactionID int           `gorm:"column:transaction_id;default:null;comment:'支付订单'" json:"transaction_id"`
	RefundAmount  string        `gorm:"column:refund_amount;size:255;default:null;comment:'退款金额'" json:"refund_amount"`
	PaymentWay    PaymentStatus `gorm:"column:payment_way;enum('wechat','alipay');default:null;comment:'支付渠道'"`
	PaymentTime   time.Time     `gorm:"column:payment_time;default:null;comment:'支付时间'" json:"payment_time"`
	Status        OrderStatus   `gorm:"column:status;type:enum('pending','processing','paid','closed','refund');default:'pending';comment:'订单状态'" json:"status"`
	RefundStatus  RefundStatus  `gorm:"column:refund_status;type:enum('pending','processing','failed','success');default:null;comment:'退款状态'" json:"refund_status"`
	Remark        string        `gorm:"column:remark;size:255;default:null;comment:'备注'" json:"remark"`
	IsDeleted     int           `gorm:"column:is_deleted;size:1;default:0;comment:'是否删除'" json:"is_deleted"`
	CreatedTime   time.Time     `gorm:"column:created_time;default:null;comment:'创建时间'" json:"created_time"`
	UpdateTime    time.Time     `gorm:"column:update_time;default:null;comment:'更新时间'" json:"update_time"`
}

func (pr *ProductOrder) BeforeCreate(db *gorm.DB) (err error) {
	pr.Status = OrderStatusPending
	pr.CreatedTime = time.Now()
	pr.IsDeleted = 0
	return
}
func CreateProductOrder(order ProductOrder) (orderNo string, err error) {
	db := models.Main()
	err = db.Create(&order).Error
	var mysqlErr *mysql.MySQLError
	if errors.As(err, &mysqlErr) && mysqlErr.Number == 1062 {
		err = gorm.ErrDuplicatedKey
	}
	orderNo = order.OrderID
	return
}

func GetTotalPageCountByUserId(userId int, status OrderStatus) (total int64, latestId int64) {
	db := models.Main()
	query := db.Model(&ProductOrder{}).Select("count(*)").Where("user_id = ? and is_deleted=0 ", userId)
	if string(status) != "" {
		query.Where("  status=?", status)
	}
	err := query.Scan(&total).Error
	if err != nil {
		logger.Error("查询订单总数失败:%v", err)
		return 0, 0
	}
	queryLatestId := db.Model(&ProductOrder{}).Select("MAX(id)").Where("user_id = ? and is_deleted=0", userId)
	if string(status) != "" {
		query.Where("  status=?", status)
	}
	err = queryLatestId.Scan(&latestId).Error
	if err != nil {
		logger.Error("查询订单最新订单失败:%v", err)
		return 0, 0
	}
	return
}

func GetOrderPage(latestId int64, offset int, size int, userId int, status OrderStatus) (orderList []ProductOrder, err error) {
	db := models.Main()
	query := db.Model(&ProductOrder{}).Select("*")
	if string(status) != "" {
		query.Where("  status=?", status)
	}
	query.Where("id<=? and user_id = ? and is_deleted=0 ", latestId, userId)
	err = query.Order("created_time desc").Offset(offset).Limit(size).Find(&orderList).Error
	if err != nil {
		logger.Error("查询订单总数失败:%v", err)
	}
	return
}

func GetOrderDetail(orderId int, userId int) (order ProductOrder, err error) {
	db := models.Main()
	err = db.Model(&ProductOrder{}).Select("*").Where("id = ? and user_id = ? and is_deleted=0 ", orderId, userId).First(&order).Error
	return
}

func GetUnPendingOrderCount() (count int, minId int) {
	db := models.Main()
	_ = db.Model(&ProductOrder{}).Select("count(*)").Where("is_deleted=0 and status=?", OrderStatusPending).Scan(&count).Error
	_ = db.Model(&ProductOrder{}).Select("MIN(id)").Where("is_deleted=0 and status =?", OrderStatusPending).Scan(&minId).Error
	return
}

func GetPendingOrderIds(minId int, limit int) (ids []int, latestId int) {
	db := models.Main()
	_ = db.Model(&ProductOrder{}).Select("id").Where("id> ? and status=? and  is_deleted=0 order by id asc", minId, OrderStatusPending).Limit(limit).Scan(&ids).Error
	latestId = ids[0]
	return
}

func batchCloseOrder(ids []int) error {
	//db := models.Main()
	//_ = db.Model(&ProductOrder{}).Select("id").Where("id> ? and status=? and  is_deleted=0 order by id asc", minId, OrderStatusPending).Limit(limit).Scan(&ids).Error
	//err = db.Model(&VerificationRecord{}).Where("expired_time < ?", time.Now()).Where("status =?", StatusPending).Update("status", StatusExpired).Error
	//if err != nil {
	//	logger.Error("批量过期验证码失败:%v", err)
	//}
	return nil
}
